<?php

namespace App\Console\Commands;

use App\Log;
use App\Models\SeoArticle;
use App\Models\SeoArticleBD;
use Illuminate\Console\Command;
use QL\QueryList;
use GuzzleHttp\Exception\RequestException;

class PMGather extends Command
{
    protected $signature = 'PMGather';
    protected $name = 'PMGather';

    protected $description = '采集人人都是产品经理文章';
    protected $link = 'http://api.woshipm.com/search/result.html';
    protected $detail = 'http://www.woshipm.com/operate/';
    protected $naiPan = 'http://www.naipan.com/open/weiyuanchuang/towei.html';
    protected $ql;

    public function __construct()
    {
        parent::__construct();
        $ql = QueryList::getInstance();
        $ql->bind('myGet',function ($url,$args = null,$otherArgs = []){
            try{
                $this->get($url,$args,$otherArgs);
            }catch(RequestException $e){
                $this->setHtml('');
                // print_r($e->getRequest());
                echo "Http Error \r\n";
            }
            return $this;
        });

        $this->ql = $ql;
    }

    public function handle()
    {
        $this->gather();
    }

    /*
     *  数据采集
     * */
    private function gather()
    {
        $keywordsData = config('pm', []);
        foreach($keywordsData as $key) {
            if(empty($key)) continue;

            # 目标链接获取
            $content = QueryList::post($this->link,[
                'key' => $key,
                'tab' => 0,
                'page' => 1,
                'idSearch' => 0,
                'sortType' => 1,
            ])->find('.article-cont')->map(function($row) {
                return [
                    'id' => $row->attr('id'),
                    'img' => preg_replace('/(http):\/\/([^\/]+)/i', 'https://www.wenxingshuju.com/pm_img', $row->find('img')->src)
                ];
            });

            foreach ($content as $item) {
                if(!isset($item['id']) || empty($item['id'])) continue;

                # 爬取详情
                $detailLink = $this->detail . $item['id'] . '.html';
                $article = $this->ql->myGet($detailLink)->rules([
                    'title' => ['title', 'text'],
                    'keywords' => ['meta[name=keywords]', 'content'],
                    'description' => ['meta[name=description]', 'content'],
                    'content' => ['.article--content', 'html', '-blockquote -.support-author -p:last -p:eq(-2) -p:eq(-3) -h3:last'],
                ])->queryData();

                if(!isset($article['title']) || empty($article['title'])) continue;

                # 处理文章标题
                $article['title'] = str_replace(" | 人人都是产品经理", '', $article['title']);

                # 处理文章内容中的图片
                $article['content'] = preg_replace_callback('/src="(.*?)"/', function($matches) {
                    return 'src="' . preg_replace('/(http):\/\/([^\/]+)/i', 'https://www.wenxingshuju.com/pm_img', $matches[1]). '" ' ;
                }, $article['content']);

                $article['catid'] = 13;
                $article['image'] = isset($item['img']) ? $item['img'] : '';
                $article['images'] = '';
                $article['summary'] = isset($item['description']) ? $item['description'] : '';
                $article['create_time'] = time();

                # 判断文章是否存在
                $isExists = SeoArticle::where('title', $article['title'])->exists();

                if($isExists) continue;     // 文章已存在跳出

                # 存储文章
                $articleId = SeoArticle::insertGetId($article);
                if($articleId) {
                    # 文章内容伪原创
//                    if($key == '视频号') {     // 当前近伪原创视频号相关内容
//                        $article['article_id'] = $articleId;
//                        $article['content'] =
//                        # 存入百度伪原创表
//                        SeoArticleBD::insert($article);
//                    }

                } else {
                    Log::logError('文章存入伪原创表失败，文章内容：', $article, 'PMGather');
                }
            }
        }
    }
}
